//
// Created by Attila Haluska on 9/8/18.
//

#include "vm/vm.h"

#include <stdbool.h>
#include <stdio.h>

#include "compiler/compiler.h"
#include "vm/value.h"
#include "vm/vm_functions.h"
#include "vm/vm_opcode.h"
#include "vm/vm_opcode_functions.h"
#include "common.h"

#if defined DEBUG_TRACE_EXECUTION
    #include "debug/debug.h"
#endif

#if defined DEBUG_TRACE_EXECUTION
static void vm_trace_execution(Vm* vm) {
    printf("== Stack ==    ");

    for (Value** slot = vm->stack; slot < vm->stack_top; slot++) {
        printf("[");
        value_print(*slot);
        printf("] ");
    }

    printf("\n");

    debug_disassemble_instruction(&vm->code, (int) (vm->ip - vm->code.code.values));
}
#endif

void vm_init(Vm* vm) {
    for (int i = 0; i < STACK_MAX; i++) {
        vm->stack[i] = NULL;
    }

    vm->ip = NULL;

    vm_reset_stack(vm);
}

void vm_free(Vm* vm) {
    if (vm != NULL) {
        vm_init(vm);
    }
}

static VmInterpretResult vm_run(Vm* vm) {
    for (;;) {
#if defined DEBUG_TRACE_EXECUTION
        vm_trace_execution(vm);
#endif

        Opcode instruction = (Opcode) vm_read_byte(vm);
        bool shouldReturn = false;
        VmInterpretResult result = VM_INTERPRET_OK;

        vm_opcode_functions[instruction](vm, &shouldReturn, &result);

        if (shouldReturn) {
            return result;
        }
    }
}

VmInterpretResult vm_interpret(Vm* vm, const char* source) {
    BytecodeArray bytecode;
    BytecodeArray* bytecode_pointer = &bytecode;

    bytecode_array_init(bytecode_pointer);
    bool compile_successful = compiler_compile_source(source, &bytecode_pointer);

    VmInterpretResult result = VM_INTERPRET_COMPILE_ERROR;

    if (compile_successful) {
        vm->code = bytecode;
        vm->ip = vm->code.code.values;
        result = vm_run(vm);
        vm_reset_stack(vm);
    }

    bytecode_array_free(bytecode_pointer);

    return result;
}
