//
// Created by Attila Haluska on 9/8/18.
//

#include "vm/vm_opcode_functions.h"

#include <stdio.h>

#include "vm/value.h"
#include "vm/vm.h"
#include "vm/vm_functions.h"

static void op_nop(Vm* vm, bool* should_return, VmInterpretResult* result) {

}

static void op_constant(Vm* vm, bool* should_return, VmInterpretResult* result) {
    Value* constant = vm_read_constant(vm);
    vm_push(vm, constant);
}

static void op_add(Vm* vm, bool* should_return, VmInterpretResult* result) {
    vm_binary_operation(vm, VM_BIN_OP_ADD, should_return, result);
}

static void op_subtract(Vm* vm, bool* should_return, VmInterpretResult* result) {
    vm_binary_operation(vm, VM_BIN_OP_SUBTRACT, should_return, result);
}

static void op_multiply(Vm* vm, bool* should_return, VmInterpretResult* result) {
    vm_binary_operation(vm, VM_BIN_OP_MULTIPLY, should_return, result);
}

static void op_divide(Vm* vm, bool* should_return, VmInterpretResult* result) {
    vm_binary_operation(vm, VM_BIN_OP_DIVIDE, should_return, result);
}

static void op_negate(Vm* vm, bool* should_return, VmInterpretResult* result) {
    if (!IS_NUMBER(vm_peek(vm, 0))) {
        *should_return = true;
        *result = VM_INTERPRET_RUNTIME_ERROR;
        vm_runtime_error(vm, "Operand must be a number");
        return;
    }

    Value* value = vm_pop(vm);

    if (IS_INTEGER(value)) {
        AS_INTEGER(value) = -AS_INTEGER(value);
    } else if (IS_DOUBLE(value)) {
        AS_DOUBLE(value) = -AS_DOUBLE(value);
    }

    vm_push(vm, value);
}

static void op_return(Vm* vm, bool* should_return, VmInterpretResult* result) {
    vm_pop(vm);
    *should_return = true;
    *result = VM_INTERPRET_OK;
}

const VmOpcodeFunction vm_opcode_functions[] = {
        op_nop,             // OP_NOP
        op_constant,        // OP_CONSTANT,
        op_add,             // OP_ADD,
        op_subtract,        // OP_SUBTRACT,
        op_multiply,        // OP_MULTIPLY,
        op_divide,          // OP_DIVIDE,
        op_negate,          // OP_NEGATE,
        op_return,          // OP_RETURN
};
